import { Component, OnInit } from "@angular/core";
import { Capability } from "../../model/capability";
import { CapabilityRequest } from "../../model/capability-request";
import { CapabilityRequestService } from "../../services/capability-request.service";
import { JsonObject } from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import { CapabilityLauncherService } from "../../services/capability-launcher.service";
import { WorkspacesComponent } from "../../workspaces.component";
import { DataRetrieverService } from "../../services/data-retriever.service";
import { PollingDataUpdaterService } from "../../services/polling-data-updater.service";
import { Subject } from "rxjs";
import { takeUntil } from "rxjs/operators";

@Component({
  selector: "app-list-of-calibratables",
  templateUrl: "./list-of-calibratables.component.html",
  styleUrls: ["./list-of-calibratables.component.scss"],
})
export class ListOfCalibratablesComponent implements OnInit {
  public calibratableRequests: Array<CapabilityRequest>;
  public capability: Capability;
  public isPaused: boolean;
  private stdCalibrationCapability: string = "std_calibration"

  private ngUnsubscribe = new Subject(); // to signal when all subscribers should unsubscribe (triggered in tear down)

  constructor(
    private capabilityRequestService: CapabilityRequestService,
    private capabilityLauncherService: CapabilityLauncherService,
    private pollingDataUpdaterService: PollingDataUpdaterService,
    private dataRetriever: DataRetrieverService,
  ) {
    this.pollingDataUpdaterService
      .getDataPoller$(this.dataRetriever.getCapabilityUrl(this.stdCalibrationCapability) )
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(this.capabilityObserver);
    this.pollingDataUpdaterService
      .getDataPoller$(this.capabilityRequestService
        .getCreatedCapabilityRequestsUrl(this.stdCalibrationCapability))
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(this.createdRequestsObserver);
  }

  ngOnInit(): void {}

  private createdRequestsObserver = {
    next: (request) => {
      this.calibratableRequests = request.created_requests;
    },
    error: (error) =>
      console.error("Error when retrieving list of created capability requests:" + error),
  };

  // Observer for capability objects
  public capabilityObserver = {
    next: (capability) => {
      this.capability = capability;
      this.isPaused = this.capability.paused
    },
    error: (error) => console.error("Error when retrieving capability:", error),
  };

  /**
   * Get metadata from capability request parameters if it exists
   * @param request Capability request
   * @return metadata Metadata section of parameters
   */
  public getMetadata(request: CapabilityRequest): any {
    if (request.parameters.hasOwnProperty("metadata")) {
      return request.parameters.metadata.valueOf();
    } else {
      return {};
    }
  }

  public submitRequest(id: string) {
    const submitCalibrationObservable = {
      next: (request) => {
        window.location.reload();
      },
      error: (error) => {
        console.error("Error when submitting calibration request:" + error);
      },
    };
    this.capabilityLauncherService.submit(id).subscribe(submitCalibrationObservable);
  }

  public togglePauseCapability(): void {
    const togglePauseCapabilityObservable = {
      next: (request) => {},
      error: (error) => {
        console.error("Error when pausing capability:" + error);
      },
    };

    let pauseAction = this.isPaused ? "unpause" : "pause"
    this.dataRetriever
      .togglePauseCapability(this.stdCalibrationCapability, pauseAction)
      .subscribe(togglePauseCapabilityObservable)
  }

}
