import { Injectable } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { Observable } from "rxjs";
import { Capability } from "../model/capability";

@Injectable({
  providedIn: "root",
})
export class DataRetrieverService {
  constructor(private httpClient: HttpClient) {}

  /**
   * Gets JSON representation of a capability given a name
   * @param name  The name of the capability
   */
  getCapability(name: string): Observable<Capability> {
    const url = `capability/${name}`;
    return this.httpClient.get<Capability>(url);
  }

  /**
   * Gets URL of a capability given a name for polling
   * @param name  The name of the capability
   */
  getCapabilityUrl(name: string): string {
    return  `capability/${name}`;
  }

  /**
   * Gets URL of a capability given a name for polling
   * @param name  The name of the capability
   */
  togglePauseCapability(name: string, pauseAction: string): Observable<any> {
    const url = `capability/${name}/${pauseAction}`;
    return this.httpClient.post<any>(url, JSON.stringify({'name': name}));
  }
}
