"""  Codifies capability steps, capability request states, execution states,
and execution priorites"""

from __future__ import annotations

from enum import Enum

# pylint: disable=C0103


class CapabilityStepType(Enum):
    """
    Enum that specifies the types of CapabilitySteps that are possible
    """

    PrepareAndRunWorkflow = "prepare-and-run-workflow"
    AwaitQA = "await-qa"
    AwaitWorkflow = "await-workflow"
    Invalid = "invalid"

    @classmethod
    def from_string(cls, string: str) -> CapabilityStepType:
        """
        Get capability step given a string

        :param string: String to convert
        :return: Capability step corresponding to given string or Invalid
        """
        strings = {
            "prepare-and-run-workflow": cls.PrepareAndRunWorkflow,
            "await-qa": cls.AwaitQA,
            "await-workflow": cls.AwaitWorkflow,
        }
        return strings.get(string, cls.Invalid)


class CapabilityRequestState(Enum):
    """
    Enum that specifies the states that a capability request can be in
    """

    Complete = 0
    Submitted = 1
    Created = 2
    Cancelled = 3
    Failed = 4


class ExecutionState(Enum):
    """
    Enum that specifies the states that a capability execution can find itself in
    """

    Complete = 0
    Created = 1  # was Ready
    ExecutingWorkflow = 2
    Waiting = 3
    Cancelled = 4
    Failed = -1


class ExecutionPriority(Enum):
    """
    Enum that specifies the priority of a particular capability execution

    TODO: These numbers are just placeholders I added on a whim, and I think this Enum should get another look in
        the future
    """

    High = 25
    Default = 50
    Low = 100
