from enum import Enum


class LocalWorkflows(Enum):
    """
    Enum that specifies which workflows can ONLY run locally - i.e. *require* lustre access
    """

    INGEST_IMAGE = "ingest_image"
    INGEST_CAL = "ingest_cal"
    CARTA = "carta"
    DOWNLOAD = "test_download"

    @classmethod
    def values(cls):
        return cls._value2member_map_

    @classmethod
    def is_local(cls, wf_name: str):
        return wf_name in cls.values()


class RemoteWorkflows(Enum):
    """
    Enum that specifies which workflows can run remotely
    """

    SECI = "vlass_seci"
    DOWNLOAD = "test_download"
    CARTA = "carta"

    @classmethod
    def values(cls):
        return cls._value2member_map_

    @classmethod
    def is_remote(cls, wf_name: str):
        return wf_name in cls.values()


class RemoteProcessingSites(Enum):
    """
    Enum specifying which remote sites are available
    """

    NAASC = "remote-cv"
    NMT = "remote"

    @classmethod
    def values(cls):
        return cls._value2member_map_
